//+------------------------------------------------------------------+
//|                                  Goertzel browser 5.5Bartels.mq4 |
//|  based on Goertzel browser 5.5 version of mladen
//|  Jan 2014 modified by Boxter
//+------------------------------------------------------------------+
/*
- UseCosineForWaves: you can choose whether you want to use 
sine or cosine for waves "reconstruction". Cosine is the right way 
to calculate Goertzel 
- UseAdditionForSteps: One question still stays as of how previous bars phases are reconstructed : 
by addition of (2*Pi/period steps) to current phase or 
by subtraction from it in cycles (this is the correct way !), 
if parameter set to true, it will add steps to current phase, 
if set to false, it will subtract steps from current phase
- UseSquaredAmp: This may be useful when adding many cycles,
but may be misleading when using just 2 or 3 cycles.
- UseTopCycles: If set to 1 you get a single sine or cosine wave which may, 
or may not coincide significantly with current "waves" in price
If set > 1 you get a composite wave of all waves <1. Use just 2 to 5 cycles at most.
- WindowSizeFuture: is the size in bars you wish to extend to the future.
- cycle list: if true it will use up to 5 cycles you choose. You don't have to use consecutive cycles. 
E.g. you cane use 1,3,5th cycle.

Goertzel browser 5.5Bartels mod:
IMPORTANT: the attached separate indis HP.mq4 and absolutely no lag lwma.mq4 MUST be added to indi folder !
- added smoothing of original time series data with Hodrick Prescott filter or Absolutly No Lag LWMA.
- in detrending function Absolutly No Lag LWMA was added as alternative to HP algo. 
Further I added a special detrending method using a linear regression of the log 
transformed smoothed time series data according to Practical Implementation of Neural 
Network based time series blog. 
- Added Bartels signifcance test function as filter for every extracted Goertzel cycle.
- modified value table a bit adding Bartels values
*/

#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers     3
#property indicator_color1      SkyBlue
#property indicator_color2      PaleVioletRed
#property indicator_color3      PaleVioletRed
#property indicator_width1      2
#property indicator_width2      2
#property indicator_width3      2

extern string info1             = "---------Main settings---------";
extern string TimeFrame         = "Current time frame";
extern int    Price             = PRICE_CLOSE;
extern string ValTypeInfo       = "0=SMA;1=HP;2=ZLlwma";
extern int    ValType           = 1;
extern int    HPsmoothPer       = 15;
extern int    ZLMAsmoothPer     = 10;
extern int    BarToCalculate    = 1;
extern int    SampleSize        = 20;
extern int    StartAtCycle      = 1;
extern int    UseTopCycles      = 15;
extern  bool   UseCosineForWaves       = true;
extern  bool   UseAdditionForSteps     = false;
extern  bool   UseSquaredAmp           = true;
extern int    MaxCycLength      = 160;
extern bool   SubtractNoise     = false;
extern bool   Interpolate       = true;

extern string info2             ="---------Display settings---------";
extern int    WindowSizePast    = 600;
extern int    WindowSizeFuture  = 200;

 color  FutureLineColor         = Orange;
 int    FutureLineStyle         = STYLE_DOT;
extern int    FutureLineDisplace = 0;
extern bool   PredictionVisible   = true;
extern bool   ListOfCyclesVisible = true;
extern bool   MultiColor        = true;
extern string ListOfCyclesID    = "GoertzelBrowser_v5.5Bartels";
 int    ListOfCyclesCorner      = 1;
 color  ListOfCyclesTextColor   = Gray;
 color  ListOfCyclesValueColor  = PaleVioletRed;
 
extern string info3             = "---------Cycle list settings---------";
extern bool   UseCycleList      = false;
extern int    Cycle1            = 1;
extern int    Cycle2            = 0;
extern int    Cycle3            = 0;
extern int    Cycle4            = 0;
extern int    Cycle5            = 0;
extern int    Offset            = 0;

extern string info4             = "Detrend & smooth data before cycle detection";
extern bool   DetrendSmoothData = false; 
extern string DTmethodInfo      = "1=HP,2=ZLMA,3=RegLnZL";
                                  //1=DPO Hodrick Prescott Filter;
                                  //2=DPO mladen absolutely no lag lwma
                                  //3=Logarit. absolutely no lag lwma to Lin.Regression
extern int    DTmethod          = 1;
extern int    DT_HPper1         = 25;
extern int    DT_HPper2         = 60;
extern int    DT_ZLper1         = 10;
extern int    DT_ZLper2         = 40;
extern int    DT_RegZLsmoothPer = 10;

extern string info5             = "Replace Amplitude by Cycle Strength";
extern bool   UseCycleStrength  = true;

// Bartels test for cycle significance (added by Boxter):
extern string info6             = "---Bartels Signifcance Test Settings---";
extern bool   FilterBartels     = true;
extern int    BartNoCycles      = 5; //considered cycles for significance
extern int    BartSmoothPer     = 5; // Renko: =0
extern double BartSigLimit      = 40; 
extern bool   SortBartels       = false;

double values[];
double goertzel[];
double goertzelUa[];
double goertzelUb[];
double goertzelFuture[];
double trend[];
double workPast[][30];
double workFuture[][30];
double cycleBartelsBuffer[];

string name;
int    ListOfCyclesWindow;

string indicatorFileName;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------

int init(){
   Comment("");
   IndicatorDigits(6);
   IndicatorBuffers(6);
   SetIndexBuffer(0, goertzel);
   SetIndexBuffer(1, goertzelUa);
   SetIndexBuffer(2, goertzelUb);
   SetIndexBuffer(3, goertzelFuture);
   SetIndexBuffer(4, values);
   SetIndexBuffer(5, trend);
      
   MaxCycLength   = MathMax(MaxCycLength,2);
 //   WindowSizePast = BartNoCycles* MaxCycLength; // new; length for significance measurements
   if (Bars<WindowSizePast){Comment("WindowSizePast not enough bars; you need ",Bars);return(0);}      
   SampleSize     = MathMax(SampleSize,2*MaxCycLength);
   BarToCalculate = MathMax(BarToCalculate,0);
   indicatorFileName = WindowExpertName();
   timeFrame      = stringToTimeFrame(TimeFrame);
   calculateValue = TimeFrame=="calculateValue";
   if (!calculateValue) ListOfCyclesID = timeFrameToString(timeFrame)+" "+ListOfCyclesID;
   string DT = "; Native;"; 
   if(ValType==1) DT = "; HP Smoothed;";
   if(ValType==2) DT = "; ZLMA Smoothed;";
   if(DetrendSmoothData) DT = "; Detrended;";
   name  = ListOfCyclesID+DT+" BarToCalc("+BarToCalculate+")";
   IndicatorShortName(name);  
   return (0);
}
 
int deinit(){
   string searchFor    = ListOfCyclesID+":";
   int    searchLength = StringLen(searchFor);
   for (int i=ObjectsTotal()-1; i>=0; i--){
      name = ObjectName(i); 
      if (StringSubstr(name,0,searchLength) == searchFor)ObjectDelete(name);
   }
   return (0);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------

#define pi 3.141592653589793238462643383279502884197169399375105820974944592

int start(){
   int counted_bars = IndicatorCounted();
   int limit,i,k,number_of_cycles;
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
   limit = MathMin(WindowSizePast-counted_bars,WindowSizePast-1);
   
   if (calculateValue || timeFrame==Period()){
      if (MultiColor && !calculateValue && trend[limit]==-1) CleanPoint(limit,goertzelUa,goertzelUb);
      ListOfCyclesWindow = WindowFind(name);
      if (DetrendSmoothData) {
         if (DTmethod==1) iDetrend_HP(values,DT_HPper1,DT_HPper2,Price,SampleSize);  
         if (DTmethod==2 )iDetrend_HP(values,DT_ZLper1,DT_ZLper2,Price,SampleSize); 
         if (DTmethod==3) iDetrend_LnZeroLagRegression(WindowSizePast,DT_RegZLsmoothPer,values) ;       
                        
      } else {
         if (ValType==0) for(i = limit; i >= 0; i--)          values[i] = iMA(NULL,0,1,0,MODE_SMA,Price,i);
         if (ValType==1) for(i = WindowSizePast; i >= 0; i--) values[i] = iCustom(NULL,0,"HP",HPsmoothPer,0,i);
         if (ValType==2) for(i = WindowSizePast; i >= 0; i--) values[i] = iCustom(NULL,0,"absolutely no lag lwma",ZLMAsmoothPer,0,i); 
      }
      // total no. of cycles
      number_of_cycles = iGoertzel(BarToCalculate,WindowSizePast,workPast,WindowSizeFuture,
                                   workFuture,UseCosineForWaves,UseAdditionForSteps,UseSquaredAmp);
  //    if (UsedTopCycles > number_of_cycles) UseTopCycles=number_of_cycles;
      if (ListOfCyclesVisible) showCycleList(number_of_cycles); 
      
      // past bar calc.  of composite wave
      for (i=0; i<WindowSizePast; i++) { // used bars past
         goertzel[i] = 0;
         if (UseCycleList){
            if (Cycle1 > 0) goertzel[i] += workPast[i][Cycle1-1];
            if (Cycle2 > 0) goertzel[i] += workPast[i][Cycle2-1];
            if (Cycle3 > 0) goertzel[i] += workPast[i][Cycle3-1];
            if (Cycle4 > 0) goertzel[i] += workPast[i][Cycle4-1];
            if (Cycle5 > 0) goertzel[i] += workPast[i][Cycle5-1];
         }else{
            for (k=StartAtCycle-1; k<StartAtCycle+UseTopCycles-1; k++) goertzel[i] += workPast[i][k];
            if (SubtractNoise)
               for (k = StartAtCycle+UseTopCycles-1; k<number_of_cycles; k++) goertzel[i] = goertzel[i]-workPast[i][k];
         }  
      }
      
      // future bar calc. of composite wave
      for (i=0; i<WindowSizeFuture; i++) {
         goertzelFuture[i] = 0;
         if (UseCycleList){
            if (Cycle1 > 0) goertzelFuture[i] += workFuture[i][Cycle1-1];
            if (Cycle2 > 0) goertzelFuture[i] += workFuture[i][Cycle2-1];
            if (Cycle3 > 0) goertzelFuture[i] += workFuture[i][Cycle3-1];
            if (Cycle4 > 0) goertzelFuture[i] += workFuture[i][Cycle4-1];
            if (Cycle5 > 0) goertzelFuture[i] += workFuture[i][Cycle5-1];   
         }else {
            for (k=StartAtCycle-1; k<StartAtCycle+UseTopCycles-1; k++) goertzelFuture[i] += workFuture[i][k];
            if (SubtractNoise)
               for (k = StartAtCycle+UseTopCycles-1; k<number_of_cycles; k++) goertzelFuture[i] = goertzelFuture[i]-workFuture[i][k];
         }
      }

      // Plot functions
      limit = WindowSizeFuture-1; int skip = -1;
      if (calculateValue) {
         skip=-0; limit+=1; 
         goertzelFuture[WindowSizeFuture-1]=goertzel[0];
         goertzelFuture[WindowSizeFuture+0]=goertzel[1];
         goertzelFuture[WindowSizeFuture+1]=goertzel[2];
         goertzelFuture[WindowSizeFuture+2]=goertzel[3]; 
      }
      else FutureLineDisplace=0;
      if (PredictionVisible)for (i=limit; i>0; i--){
         string cname = ListOfCyclesID+":l:"+i;
         ObjectCreate(cname,OBJ_TREND,ListOfCyclesWindow,0,0);
         ObjectSet(cname,OBJPROP_TIME1, Time[0]+Period()*60*(WindowSizeFuture-i+skip)  -FutureLineDisplace);
         ObjectSet(cname,OBJPROP_TIME2, Time[0]+Period()*60*(WindowSizeFuture-i+skip+1)-FutureLineDisplace);
         ObjectSet(cname,OBJPROP_PRICE1,goertzelFuture[i]);
         ObjectSet(cname,OBJPROP_PRICE2,goertzelFuture[i-1]);
         ObjectSet(cname,OBJPROP_RAY,false);
         ObjectSet(cname,OBJPROP_COLOR,FutureLineColor);
         ObjectSet(cname,OBJPROP_STYLE,FutureLineStyle);
      }
      
      for (i=WindowSizePast-1; i>=0; i--){
         goertzelUa[i] = EMPTY_VALUE;
         goertzelUb[i] = EMPTY_VALUE;
         trend[i]      = trend[i+1];
         if (goertzel[i] > goertzel[i+1]) trend[i] =  1;
         if (goertzel[i] < goertzel[i+1]) trend[i] = -1;
         if (MultiColor && !calculateValue && trend[i]==-1) PlotPoint(i,goertzelUa,goertzelUb,goertzel);
      }           
      SetIndexDrawBegin(0,Bars-WindowSizePast+1);
      return(0);
   }

   // MTF calc.
   limit = MathMin(Bars-1,WindowSizePast*timeFrame/Period());
   if (MultiColor && trend[limit]==-1) CleanPoint(limit,goertzelUa,goertzelUb);
   for (i=limit;i>=0; i--){
      int y  = iBarShift(NULL,timeFrame,Time[i]);

      goertzel[i] =   iCustom(NULL,timeFrame,indicatorFileName,"","calculateValue",Price,"",ValType,HPsmoothPer,
                      ZLMAsmoothPer,BarToCalculate,SampleSize,StartAtCycle,UseTopCycles,UseCosineForWaves,UseAdditionForSteps,UseSquaredAmp,
                      MaxCycLength,SubtractNoise,Interpolate,"",WindowSizePast,WindowSizeFuture,Period()*60,PredictionVisible,
                      ListOfCyclesVisible,false,ListOfCyclesID,"",UseCycleList,Cycle1,Cycle2,Cycle3,Cycle4,Cycle5,Offset,
                      "",DetrendSmoothData,"",DT_HPper1,DT_HPper2,DT_ZLper1,DT_ZLper2,DT_RegZLsmoothPer,"",UseCycleStrength,
                      "",FilterBartels,BartNoCycles,BartSmoothPer,BartSigLimit,SortBartels,0,y);
                      
      trend[i]      = iCustom(NULL,timeFrame,indicatorFileName,"","calculateValue",Price,"",ValType,HPsmoothPer,
                      ZLMAsmoothPer,BarToCalculate,SampleSize,StartAtCycle,UseTopCycles,UseCosineForWaves,UseAdditionForSteps,UseSquaredAmp,
                      MaxCycLength,SubtractNoise,Interpolate,"",WindowSizePast,WindowSizeFuture,Period()*60,PredictionVisible,
                      ListOfCyclesVisible,false,ListOfCyclesID,"",UseCycleList,Cycle1,Cycle2,Cycle3,Cycle4,Cycle5,Offset,
                      "",DetrendSmoothData,"",DT_HPper1,DT_HPper2,DT_ZLper1,DT_ZLper2,DT_RegZLsmoothPer,"",UseCycleStrength,
                      "",FilterBartels,BartNoCycles,BartSmoothPer,BartSigLimit,SortBartels,5,y);
      goertzelUa[i] = EMPTY_VALUE;
      goertzelUb[i] = EMPTY_VALUE;
      if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue; interpolate(goertzel,iTime(NULL,timeFrame,y),i);
   }
   if (MultiColor) for (i=limit;i>=0;i--) if (trend[i]==-1) PlotPoint(i,goertzelUa,goertzelUb,goertzel);
   SetIndexDrawBegin(0,Bars-WindowSizePast*(timeFrame/Period())+1);
   return (0);
}

//+------------------------------------------------------------------
//| sub functions
//+------------------------------------------------------------------

//+------------------  Interpolate function-----------------------------------------------------

void interpolate(double& buffer[], datetime time, int i){
   for (int n = 1; (i+n) < Bars && Time[i+n] >= time; n++) continue;
   if (buffer[i] == EMPTY_VALUE || buffer[i+n] == EMPTY_VALUE) n=-1;
               double increment = (buffer[i+n] - buffer[i])/ n;
   for (int k = 1; k < n; k++)     buffer[i+k] = buffer[i] + k*increment;
}

//+-------------- Goertzel function-----------------------------------------------------

double  goeWork1[];
double  goeWork2[];
double  goeWork3[];
double  goeWork4[];
double  cycleLengthBuffer[];
double  cycleAmplitBuffer[];
double  cyclePhaseBuffer[];

int iGoertzel(int forBar, int numBarsPast, double& goeWorkPast[][], int numBarsFuture, double& goeWorkFuture[][], bool useCosine = true, bool useAddition = true, bool squaredAmp=true){
   int sample = MathMin(Bars-forBar,SampleSize);
   if (ArraySize(goeWork4)!=sample+1){
       ArrayResize(goeWork1,sample+1); ArrayResize(goeWork2,sample+1);
       ArrayResize(goeWork3,sample+1); ArrayResize(goeWork4,sample+1);
   }
   if (ArraySize(cycleLengthBuffer)!=MaxCycLength+1){
      ArrayResize(cycleLengthBuffer,MaxCycLength+1);
      ArrayResize(cycleAmplitBuffer,MaxCycLength+1);
      ArrayResize(cyclePhaseBuffer ,MaxCycLength+1);
   }
   if (ArrayRange(goeWorkPast,0)  !=numBarsPast)   ArrayResize(goeWorkPast  ,numBarsPast);
   if (ArrayRange(goeWorkFuture,0)!=numBarsFuture) ArrayResize(goeWorkFuture,numBarsFuture);

   double temp1 =  values[forBar + sample-1];
   double temp2 = (values[forBar] - temp1) / (sample-1);
   double temp3;
   for (int k = sample; k>0; k--) {
       goeWork4[k] = values[forBar+k-1] - (temp1 + temp2*(sample-k));
       goeWork3[k] = 0;
   }
   goeWork3[0]=0;

   for (k = 2; k <= MaxCycLength; k++){
       double w = 0;
       double x = 0;
       double y = 0;
       double z = MathPow(k, -1);
       temp1 = 2.0 * MathCos(2.0 * pi * z);
       for (int i = sample; i > 0; i--){
           w = temp1 * x - y + goeWork4[i];
           y = x;
           x = w;
       }
       temp2 = x - y / 2.0 * temp1;
       if (temp2 == 0.0) temp2 = 0.0000001;
       temp3 = y * MathSin(2.0 * pi * z);

       if (UseCycleStrength){ //Cycle Strength  = Cycle Amplitude/ Cycle Length
                              //as described in Decoding the Hidden Market Rhythm. 
                              //For trading it is more important to know which cycle 
                              //has the biggest influence to drive the price per bar, 
                              //and not only which cycle has the highest amplitude!
          if(squaredAmp==true)goeWork1[k] = (MathPow(temp2,2)+MathPow(temp3,2))/k;
          else                goeWork1[k] = (MathSqrt(MathPow(temp2,2)+MathPow(temp3,2)))/k;
          goeWork2[k] = MathArctan(temp3/temp2);
       } else {
          if(squaredAmp==true)goeWork1[k] =  MathPow(temp2,2)+MathPow(temp3,2);
          else                goeWork1[k] =  MathSqrt(MathPow(temp2,2)+MathPow(temp3,2));
          goeWork2[k] = MathArctan(temp3/temp2);
       }

       if (temp2 < 0.0)      goeWork2[k] += pi;
       else if (temp3 < 0.0) goeWork2[k] += 2.0 * pi;
   }
   
   for (k = 3; k < MaxCycLength; k++)
       if (goeWork1[k] > goeWork1[k+1] && goeWork1[k] > goeWork1[k-1]) goeWork3[k] = k * MathPow(10, -4);
       else  goeWork3[k] = 0.0;

   //    extract cycles
   int number_of_cycles = 0;
   for (i = 0; i<MaxCycLength+2; i++) if (goeWork3[i] > 0.0){
       cycleLengthBuffer[number_of_cycles] = MathRound(MathPow(10,4) * goeWork3[i]);
       cycleAmplitBuffer[number_of_cycles] = goeWork1[i];
       cyclePhaseBuffer [number_of_cycles] = goeWork2[i];
       number_of_cycles++;
   }
   
   //    order cycles acc. largest amplitude or cycle strength
   for (i = 0; i < number_of_cycles-1; i++)
       for (k = i+1; k < number_of_cycles;   k++)
           if (cycleAmplitBuffer[k] > cycleAmplitBuffer[i]){
               y = cycleAmplitBuffer[i];
               w = cycleLengthBuffer[i];
               x = cyclePhaseBuffer[i];
               cycleAmplitBuffer[i] = cycleAmplitBuffer[k];
               cycleLengthBuffer[i] = cycleLengthBuffer[k];
               cyclePhaseBuffer[i]  = cyclePhaseBuffer[k];
               cycleAmplitBuffer[k] = y;
               cycleLengthBuffer[k] = w;
               cyclePhaseBuffer[k]  = x;
   }
   
   // Execute Bartels test for cycle significance
   if (ArrayRange(cycleBartelsBuffer,0) != number_of_cycles) ArrayResize(cycleBartelsBuffer,number_of_cycles);
   ArrayInitialize(cycleBartelsBuffer,EMPTY_VALUE); 
   iBartelsCycleTest(cycleLengthBuffer,number_of_cycles, cycleBartelsBuffer);
 //  for (i=0;i<number_of_cycles;i++) Print("Old cycleBartelsBuffer[",i,"]=",cycleBartelsBuffer[i]);   

   // order cycles acc. best Bartels prob.
   if (FilterBartels){   //Filter significant cycles
      int no_Bcycles=0, v;
      for (i = 0; i < number_of_cycles-1; i++) if (cycleBartelsBuffer[i]>BartSigLimit){
         cycleAmplitBuffer[no_Bcycles]=cycleAmplitBuffer[i];
         cycleLengthBuffer[no_Bcycles]=cycleLengthBuffer[i];
         cyclePhaseBuffer[no_Bcycles] = cyclePhaseBuffer[i];
         cycleBartelsBuffer[no_Bcycles] = cycleBartelsBuffer[i];
         no_Bcycles++;   
      }
      
      if (no_Bcycles==0)number_of_cycles=0;
      else {
         ArrayResize(cycleLengthBuffer,no_Bcycles);
         ArrayResize(cycleAmplitBuffer,no_Bcycles);
         ArrayResize(cyclePhaseBuffer ,no_Bcycles);
         ArrayResize(cycleBartelsBuffer,no_Bcycles);
         number_of_cycles = no_Bcycles;
      }      
      
      //Sort Bartels
      if (SortBartels && number_of_cycles >1)
         for (i = 0; i < number_of_cycles-1; i++)
           for (k = i+1; k < number_of_cycles;   k++)
            if (cycleBartelsBuffer[k] > cycleBartelsBuffer[i]){  // > because inverse Bartels percentage used
               y = cycleAmplitBuffer[i];
               w = cycleLengthBuffer[i];
               x = cyclePhaseBuffer[i];
               v = cycleBartelsBuffer[i];
               cycleAmplitBuffer[i] = cycleAmplitBuffer[k];
               cycleLengthBuffer[i] = cycleLengthBuffer[k];
               cyclePhaseBuffer[i]  = cyclePhaseBuffer[k];
               cycleBartelsBuffer[i]= cycleBartelsBuffer[k];
               cycleAmplitBuffer[k] = y;
               cycleLengthBuffer[k] = w;
               cyclePhaseBuffer[k]  = x;
               cycleBartelsBuffer[k]= v;
      } 
   }

   //    calculate waves
   for (i=0; i<number_of_cycles; i++) {
       double amplitude = cycleAmplitBuffer[i];
       double phase     = cyclePhaseBuffer[i];
       int    cycle     = cycleLengthBuffer[i];
       double sign      = 1;
       if (!useAddition) sign=-1;
       
       for (k=0; k<numBarsPast; k++){
           if (useCosine)goeWorkPast[k][i] = amplitude * MathCos(phase+sign*k*2.0*pi/cycle);
           else          goeWorkPast[k][i] = amplitude * MathSin(phase+sign*k*2.0*pi/cycle);
       }
       sign *= -1;
       for (k=0; k<numBarsFuture; k++){
           if (useCosine)goeWorkFuture[numBarsFuture-k-1][i] = amplitude * MathCos(phase+sign*k*2.0*pi/cycle);
           else          goeWorkFuture[numBarsFuture-k-1][i] = amplitude * MathSin(phase+sign*k*2.0*pi/cycle);
       }
   }   
   
   return(number_of_cycles);
}

//+------------------------------------------------------------------
//|  functions  Display Cycle List  
//+------------------------------------------------------------------

void showCycleList(int number_of_cycles){
      deleteCycleList();
      //    normalize amplitude values display to < 100
      
      double max   = cycleAmplitBuffer[ArrayMaximum(cycleAmplitBuffer,number_of_cycles)];
      double coeff = MathMax(MathCeil(MathLog(max)/MathLog(10))-2,-4);

      showCycle(1,number_of_cycles+2,-1,0,0,0); // title
      for (int i=0; i< number_of_cycles; i++) showCycle(i+2,number_of_cycles+2,cycleLengthBuffer[i],
                                                        cycleAmplitBuffer[i]/MathPow(10,coeff),
                                                        MathMod(cyclePhaseBuffer[i],2.0*pi)*180/pi, 
                                                        cycleBartelsBuffer[i]);
      showCycle(i+2,number_of_cycles+2,-2,coeff,0,0);  //number_of_cycles+2 because of additional title line and coeff line         
}

void deleteCycleList(){
   string searchFor    = ListOfCyclesID+":";
   int    searchLength = StringLen(searchFor);
   for (int i=ObjectsTotal()-1; i>=0; i--){
      string name = ObjectName(i); 
      if (StringSubstr(name,0,searchLength) == searchFor)ObjectDelete(name);
   }
}

double xpos1[] = {169,147,102,52,10}; // with titles
double xpos2[] = {10,52,102,147,169}; // with titles

void showCycle(int cycleNo, int totalCycles, double period, double amplitude, double phase, double BP){
// adapted and extended by Boxter
   double xpos[5]; // former [6]; incl. Bartels value
   int    ypos; 
   string Amp = "Amp"; if (UseCycleStrength) Amp = "CySt"; 
      
   if (ListOfCyclesCorner== 2 || ListOfCyclesCorner==3) 
         ypos = (totalCycles-cycleNo+1)*10;
   else  ypos = cycleNo*10;          
   if (ListOfCyclesCorner== 1 || ListOfCyclesCorner==3) //1=right upper corner, 3 right lower corner
         ArrayCopy(xpos,xpos1);
   else  ArrayCopy(xpos,xpos2);
   
   if (period==-1){    //title
      setElement(cycleNo+"-0",xpos[0],ypos,ListOfCyclesTextColor,"Rnk");
      setElement(cycleNo+"-1",xpos[1],ypos,ListOfCyclesTextColor,"Prd");
      setElement(cycleNo+"-2",xpos[2],ypos,ListOfCyclesTextColor,"Brt"); // Bartels probability
      setElement(cycleNo+"-3",xpos[3],ypos,ListOfCyclesTextColor,Amp);  
      setElement(cycleNo+"-4",xpos[4],ypos,ListOfCyclesTextColor,"Phs"); 
   }  
   
   if (period >0){         
      setElement(cycleNo+"-0",xpos[0],ypos,ListOfCyclesValueColor,cycleNo-1);
      setElement(cycleNo+"-1",xpos[1],ypos,ListOfCyclesValueColor,DoubleToStr(period,0));
      setElement(cycleNo+"-2",xpos[2],ypos,ListOfCyclesValueColor,DoubleToStr(BP,2)); // Bartels probability
      setElement(cycleNo+"-3",xpos[3],ypos,ListOfCyclesValueColor,DoubleToStr(amplitude,4));            
      setElement(cycleNo+"-4",xpos[4],ypos,ListOfCyclesValueColor,DoubleToStr(phase,2));
   }      
      
   if (period==-2)
      if (ListOfCyclesCorner== 1 || ListOfCyclesCorner==3)
            setElement(cycleNo+"-5",xpos[4],ypos,ListOfCyclesTextColor ,"* amplitude multiplier : 10^"+DoubleToStr(amplitude,0),8);
      else  setElement(cycleNo+"-5",xpos[0],ypos,ListOfCyclesTextColor ,"* amplitude multiplier : 10^"+DoubleToStr(amplitude,0),8);  
}

void setElement(string element, int x, int y, color ecolor=Silver, string text = "", int fontsize = 8, string fontname="Arial", int angle=0){
   string name = ListOfCyclesID+":"+element+" "+Symbol();
   ObjectCreate(name, OBJ_LABEL, ListOfCyclesWindow, 0, 0);
      ObjectSet(name, OBJPROP_XDISTANCE, x);
      ObjectSet(name, OBJPROP_YDISTANCE, y);
      ObjectSet(name, OBJPROP_BACK, FALSE);
      ObjectSet(name, OBJPROP_CORNER, ListOfCyclesCorner);
      ObjectSet(name, OBJPROP_ANGLE, angle);
      ObjectSetText(name, text, fontsize, fontname, ecolor);
}

//+----------------- MTF function

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

int stringToTimeFrame(string tfs){
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

string timeFrameToString(int tf){
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

string stringUpperCase(string str){
   string   s = str;
   for (int length=StringLen(str)-1; length>=0; length--){
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

//+----------------- Plot function

void CleanPoint(int i,double& first[],double& second[]){
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[]){
   if (first[i+1] == EMPTY_VALUE){
      if (first[i+2] == EMPTY_VALUE) {
          first[i]   = from[i];
          first[i+1] = from[i+1];
          second[i]  = EMPTY_VALUE;
      }else {
          second[i]   =  from[i];
          second[i+1] =  from[i+1];
          first[i]    = EMPTY_VALUE;
      }
   }else{
      first[i]  = from[i];
      second[i] = EMPTY_VALUE;
   }
}

//--------------Hodrick Prescott Detrending--------------------------+

double sourceValues[];
double calcValues1[];
double calcValues2[];
double a[],b[],c[];

void iDetrend_HP(double& output[], int period1, int period2, int price, int calcBars){
   calcBars = MathMin(WindowSizePast,MathMax(calcBars+period1+period2,500));
   if (calcBars > WindowSizePast) calcBars = WindowSizePast;
   if (ArraySize(sourceValues) != calcBars) {
       ArrayResize(sourceValues,calcBars);
       ArrayResize(calcValues1 ,calcBars);
       ArrayResize(calcValues2 ,calcBars);
       ArrayResize(a           ,calcBars);
       ArrayResize(b           ,calcBars);
       ArrayResize(c           ,calcBars);
   }  
   for(int i=calcBars; i>=0; i--)  sourceValues[i]=iMA(NULL,0,1,0,MODE_SMA,price,i);
   if (DTmethod==1){
      iHp(sourceValues,0.0625/MathPow(MathSin(pi/period1),4),calcBars,calcValues1,true);
      iHp(sourceValues,0.0625/MathPow(MathSin(pi/period2),4),calcBars,calcValues2,true);
      for(i=calcBars; i>=0; i--) output[i] = calcValues1[i]-calcValues2[i];
   }
   if (DTmethod==2)      
      for(i=calcBars; i>=0; i--) output[i] =iCustom(NULL,0,"absolutely no lag lwma",period1,price,0,i)-iCustom(NULL,0,"absolutely no lag lwma",period2,price,0,i);
}

void iHp(double data[], double lambda, int nobs, double& output[], bool ret=true){
   double H1 =0, H2 =0, H3 =0, H4 =0, H5 =0;
   double HH1=0, HH2=0, HH3=0, HH4=0, HH5=0;
   double HB,HC;
   double Z;
   if (nobs <= 5) return;
   
   ArrayCopy(output,data); 
   a[0]= 1.0+lambda;
   b[0]=-2.0*lambda;
   c[0]=     lambda;
   
   for(int i=1;i<nobs-2;i++){
      a[i]= 6.0*lambda+1.0;
      b[i]=-4.0*lambda;
      c[i]=     lambda;                                             
   }

   a[1]      = 1.0+lambda*5.0;
   a[nobs-1] = 1.0+lambda;
   a[nobs-2] = 1.0+lambda*5.0;
   b[nobs-2] =-2.0*lambda;
   b[nobs-1] = 0.0;
   c[nobs-2] = 0.0;
   c[nobs-1] = 0.0;
      
   for (i=0;i<nobs;i++){
      Z=a[i]-H4*H1-HH5*HH2;
      if (Z==0) break;
      HB   = b[i];
      HH1  = H1;
      H1   = (HB-H4*H2)/Z;
      b[i] = H1;

      HC   = c[i];
      HH2  = H2;
      H2   = HC/Z;
      c[i] = H2;

      a[i] = (output[i]-HH3*HH5-H3*H4)/Z;
      HH3  = H3;
      H3   = a[i];
      H4   = HB-H5*HH1;
      HH5  = H5;
      H5   = HC;
   }
   H2 = 0;
   H1 = a[nobs-1];
   output[nobs-1]=H1;
   for (i=nobs-2; i>=0; i--){
      output[i]=a[i]-b[i]*H1-c[i]*H2;
      H2=H1;
      H1=output[i];
   }
   if (ret==false) for(i=0; i<nobs; i++) output[i] -= data[i];
}

//----------------------Bartels Cycle Significance Test function-------------------------------
// BPeriods[i] = List with all different Goertzel Periods 
// G= No. of Goertzel cycles
// NoSigCycles = no. of considered cycles for significance per cycle curve (usually 5, extern variable)
// cycleBartelsBuffer = Bartels Probability: Used for statistical significance testing;
// check http://www.cyclesresearchinstitute.org/cycles-general/bartel.pdf

double TFvalues[];                 // time series data per cycle period
void iBartelsCycleTest(double BPeriods[], int G, double& BartelsProb[]){
   int i,t,k;
   if (ArrayRange(BPeriods,0) !=G)     ArrayResize(BPeriods,G); 
   if (ArrayRange(BartelsProb,0) !=G)  ArrayResize(BartelsProb,G);
     
   for ( k=0; k<G; k++) {
      // Extract time series data per cycle considered for significance
      int bpi = MathRound(BPeriods[k]);
      
      if (ArrayRange(TFvalues,0) != bpi*BartNoCycles) {ArrayResize(TFvalues,bpi*BartNoCycles);} // incl. 0 variable
      ArrayInitialize(TFvalues,EMPTY_VALUE);   
      iDetrend_LnZeroLagRegression(bpi*BartNoCycles,BartSmoothPer,TFvalues) ; 
      BartelsProb[k] = (1-iBartelsProb(bpi,BartNoCycles))*100; //TFvalues, reverse percentage as Bartels value
   } 
}  

double teta[],vsin[],vcos[],CoeffA[],CoeffB[],IndAmplit[];
double iBartelsProb(int n,int N) {  //double X[],n=period or no. bars per cycle; X[][]=time series values  
   int t,i;
   double a,b;
   double AvgCoeffA=0,AvgCoeffB=0,AvgIndAmplit=0,AvgAmpl,ExptAmpl,ARatio,BP;
      
   if (ArrayRange(teta,0) !=n)   ArrayResize(teta,n);    ArrayInitialize(teta,0);
   if (ArrayRange(vsin,0) !=n)   ArrayResize(vsin,n);    ArrayInitialize(vsin,0);
   if (ArrayRange(vcos,0) !=n)   ArrayResize(vcos,n);    ArrayInitialize(vcos,0);
   if (ArrayRange(CoeffA,0) !=N) ArrayResize(CoeffA,N);  ArrayInitialize(CoeffA,0);
   if (ArrayRange(CoeffB,0) !=N) ArrayResize(CoeffB,N);  ArrayInitialize(CoeffB,0);  
   if (ArrayRange(IndAmplit,0)!=N)ArrayResize(IndAmplit,N); ArrayInitialize(IndAmplit,0);  
   
   // Calculation of sin and cos values per cycle period  
   for (i=0; i<n; i++){ 
      teta[i]=1.0*(i+1)/n*2*pi; 
      vsin[i]= MathSin(teta[i]);
      vcos[i]= MathCos(teta[i]); 
   }
   
   // Calculate individual coefficients A, B and individual amplitude 
   // (Bartels cycles are calculated either with sin or cos waves. For measuring cycle significance
   // only the no. of considered cycle periods is important, not the kind of cycle calculation. 
   // Therefore the combined sin/cos usage for calculating significance coeffs is ok)
   for (t=0; t<N; t++){  
      for(i=0; i<n; i++){ 
         CoeffA[t] += vsin[i]*TFvalues[t*n+i];  
         CoeffB[t] += vcos[i]*TFvalues[t*n+i];  
      } 
      IndAmplit[t] = MathPow(CoeffA[t],2) + MathPow(CoeffB[t],2);
   }   
   
   // Average coefficients and average individual amplitude 
   for (t=0; t<N; t++){ 
      AvgCoeffA      += CoeffA[t];
      AvgCoeffB      += CoeffB[t];
      AvgIndAmplit   += IndAmplit[t];
   }
   AvgCoeffA = AvgCoeffA/N; 
   AvgCoeffB = AvgCoeffB/N;                
   AvgAmpl = MathSqrt( MathPow(AvgCoeffA,2)+MathPow(AvgCoeffB,2) ); // Average amplitude
   
   AvgIndAmplit = MathSqrt(AvgIndAmplit/N); //Avg. Individual Ampl. or Avg.Vector
   ExptAmpl = AvgIndAmplit / MathSqrt(1.0*N);  // Expected Amplitude
   
   ARatio = AvgAmpl/ExptAmpl;
   BP=1/MathExp(MathPow(ARatio,2));
   return(BP);
}  

//--------------Logarithmic ZeroLagMA Values Regression Detrending --------------------------+
double RegValue[];
void iDetrend_LnZeroLagRegression(int ConsidBars, int ZLper, double& TFval[]) { 
   int i;
   if (ArrayRange(RegValue,0) != ConsidBars) ArrayResize(RegValue,ConsidBars); 
   ArrayInitialize(TFval,EMPTY_VALUE);
   ArrayInitialize(RegValue,EMPTY_VALUE);
   
 //   ln of AbsNoLagLWMA ;centered MA developed by mladen - used for smoothing time series; 
   for (i=ConsidBars-1;i>=0;i--){
      TFval[i]=MathLog(iCustom(NULL,0,"absolutely no lag lwma",ZLper,0,0,i));
   }  
       
//linear regression calc.
   double value=TFval[0];
   double a,b,c;
   double sumy=value;
   double sumx=0.0;
   double sumxy=0.0;
   double sumx2=0.0;
   for(i=1; i<ConsidBars; i++){
      value=TFval[i];
      sumy+=value;
      sumxy+=value*i;
      sumx+=i;
      sumx2+=i*i;
   }
   c=sumx2*ConsidBars-sumx*sumx;
   if(c==0.0) return;
   b=(sumxy*ConsidBars-sumx*sumy)/c;
   a=(sumy-sumx*b)/ConsidBars;
   double LR.price.2=a;
   double LR.price.1=a+b*ConsidBars;
   for(i=0; i<=ConsidBars; i++)RegValue[i]=a+b*i;// Print("RegValue[",i,"]=",RegValue[i]);}
   for(i=0; i<=ConsidBars; i++)TFval[i]=TFval[i]-RegValue[i];
   return(0);   
}
//  "absolutely no lag lwma" by mladen - Forex TSD
//  http://www.forex-tsd.com/elite-section/3580-elite-indicators-252.html#post381102


 
   